<?php

namespace Drupal\ui_patterns_field_formatters\Plugin\Derivative;

use Drupal\Core\TypedData\DataDefinitionInterface;

/**
 * Provides entity field block definitions for every field.
 *
 * @internal
 *   Plugin derivers are internal.
 *
 *   @SuppressWarnings(PHPMD.CamelCaseParameterName)
 *   @SuppressWarnings(PHPMD.CamelCaseVariableName)
 *   @SuppressWarnings(PHPMD.LongClassName)
 *   @SuppressWarnings(PHPMD.LongVariable)
 */
class FieldPropertiesSourcePropDeriver extends FieldPropertiesSourceDeriverBase {

  /**
   * Map the pop type SDC with the field type Drupal.
   *
   * @return array
   *   The complete property mapping between Drupal field and SDC.
   */
  protected function getPropTypeToFieldTypeMap(): array {
    $returned = [];
    $typed_data_definitions = $this->typedDataManager->getDefinitions();
    foreach ($typed_data_definitions as $typed_data_definition) {
      $data_type_id = $typed_data_definition["id"];
      $returned[$data_type_id] = [];
    }
    // @todo the values here in the table are totally incomplete
    // MISSING : @see https://www.drupal.org/docs/drupal-apis/entity-api/fieldtypes-fieldwidgets-and-fieldformatters
    // map
    $prop_mapping = [
      'string' => [
        'changed',
        'created',
        'datetime',
        'datetime_iso8601',
        'daterange',
        'email',
        'entity_reference',
        'language',
        'password',
        'path',
        'string',
        'string_long',
        'telephone',
        'text',
        'text_long',
        'text_with_summary',
        'timestamp',
        'uri',
        'uuid',
      ],
      'enum_list' => [
        'list_float',
        'list_integer',
        'list_string',
      ],
      'number' => ['integer', 'decimal', 'float', 'duration', 'file', 'image'],
      'boolean' => ['boolean'],
      'url' => ['uri', 'link', 'file_uri'],
    ];
    foreach ($prop_mapping as $ui_patterns_pop_type => $field_prop_types) {
      foreach ($field_prop_types as $field_prop_type) {
        if (array_key_exists($field_prop_type, $returned)) {
          $returned[$field_prop_type][] = $ui_patterns_pop_type;
        }
      }
    }
    return $returned;
  }

  /**
   * {@inheritdoc}
   */
  protected function getPropTypesDeriver(DataDefinitionInterface $property_definition): array {
    $dataTypeToPropType = $this->getPropTypeToFieldTypeMap();
    $dataType = $property_definition->getDataType();
    if (array_key_exists($dataType, $dataTypeToPropType)) {
      return $dataTypeToPropType[$dataType];
    }
    return [];
  }

}
