<?php

declare(strict_types=1);

namespace Drupal\ui_patterns_field_formatters\Plugin\Field\FieldFormatter;

use Drupal\Core\Field\FieldItemListInterface;

/**
 * Plugin implementation of the 'component_each' formatter.
 *
 * Field types are altered in
 * ui_patterns_field_formatters_field_formatter_info_alter().
 *
 * @FieldFormatter(
 *   id = "ui_patterns_each",
 *   label = @Translation("Component for each (UI Patterns)"),
 * )
 *
 *   @SuppressWarnings(PHPMD.CamelCaseParameterName)
 *   @SuppressWarnings(PHPMD.CamelCaseVariableName)
 *   @SuppressWarnings(PHPMD.LongClassName)
 *   @SuppressWarnings(PHPMD.LongVariable)
 */
class ComponentEachFormatter extends ComponentFormatterBase {

  /**
   * {@inheritdoc}
   */
  public function viewElements(FieldItemListInterface $items, $langcode) {
    $build = [];
    foreach ($items as $field_item_index => $field_item) {
      $this->setComponentSettingsAltered([
        'field_index' => $field_item_index,
        'lang_code' => $langcode,
        'items' => $items,
      ]);
      $build[] = $this->buildComponentRenderable($this->getComponentConfiguration()['component_id'], $this->getComponentSourceContexts());
    }

    return $build;
  }

}
