<?php

declare(strict_types=1);

namespace Drupal\ui_patterns_field_formatters\Plugin\UiPatterns\Source;

use Drupal\Core\Entity\EntityFieldManagerInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Entity\EntityTypeRepositoryInterface;
use Drupal\Core\Entity\FieldableEntityInterface;
use Drupal\Core\Field\BaseFieldDefinition;
use Drupal\Core\Field\FieldDefinitionInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Logger\LoggerChannelTrait;
use Drupal\Core\Plugin\Context\ContextRepositoryInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Entity reference form builder trait.
 *
 * @todo this class is not yet implemented, needed to change the feature of
 * entity reference deriver to settings of ui patterns field formatter.
 *
 * @SuppressWarnings(PHPMD.CamelCaseParameterName)
 * @SuppressWarnings(PHPMD.CamelCaseVariableName)
 * @SuppressWarnings(PHPMD.LongClassName)
 * @SuppressWarnings(PHPMD.LongVariable)
 */
abstract class EntityReferenceFormBase extends FieldPropertiesSourceBase {

  use LoggerChannelTrait;

  /**
   * {@inheritdoc}
   */
  public static function create(
    ContainerInterface $container,
    array $configuration,
    $plugin_id,
    $plugin_definition
  ) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('context.repository'),
      $container->get('entity_type.manager'),
      $container->get('entity_type.repository'),
      $container->get('entity_field.manager')

    );
  }

  /**
   * {@inheritdoc}
   */
  public function __construct(
    array $configuration,
    $plugin_id,
    $plugin_definition,
    protected ContextRepositoryInterface $contextRepository,
    protected EntityTypeManagerInterface $entityTypeManager,
    protected EntityTypeRepositoryInterface $entityTypeRepository,
    protected EntityFieldManagerInterface $entityFieldManager,
  ) {
    parent::__construct($configuration, $plugin_id, $plugin_definition, $contextRepository, $entityTypeManager);
  }

  /**
   * Callback to build field formatter form.
   *
   * @param array $form
   *   The source plugin settings form.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *
   * @return void|null
   *
   * @throws \Drupal\Component\Plugin\Exception\PluginException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  private function buildFieldEntityReferenceForm(array &$form, FormStateInterface $form_state) {
    $plugin_definition = $this->getPluginDefinition();
    $field_definition = $plugin_definition['ui_patterns_field_formatter']['field_definition'];

    if (!$field_definition instanceof FieldDefinitionInterface) {
      return NULL;
    }
    if ($field_definition->getType() !== 'entity_reference') {
      return NULL;
    }
    $settings = $field_definition->getSettings();
    $entity_type = $this->entityTypeManager->getDefinition($settings['target_type']);
    // Fail with an exception for non-fieldable entity types.
    if (!$entity_type->entityClassImplements(FieldableEntityInterface::class)) {
      return NULL;
    }
    $entity_type_labels = $this->entityTypeRepository->getEntityTypeLabels();

    // Get the bundle of entity.
    $bundles = $settings['handler_settings']['target_bundles'] ?? [$settings['target_type']];
    $source_name = 'props';
    if (!empty($plugin_definition['prop_types'][0]) && $plugin_definition['prop_types'][0] === 'slot') {
      $source_name = 'slots';
    }
    // Set parent can be render as main property.
    // $this->derivatives[$derivative['id']]['ui_patterns_field_formatter']['main_property']=TRUE;.
    foreach ($bundles as $bundle) {
      $field_definitions_reference = $this->entityFieldManager->getFieldDefinitions($settings['target_type'], $bundle);
      foreach ($field_definitions_reference as $field_name => $field_info_reference) {
        if ($field_info_reference instanceof BaseFieldDefinition && $field_info_reference->getType() === 'entity_reference') {
          /** @var   \Drupal\Core\Field\TypedData\FieldItemDataDefinition $prop_def */
          $prop_def = $field_info_reference->getFieldStorageDefinition()
            ->getItemDefinition();
          if ($prop_def->getSetting('target_type') === $settings['target_type']) {
            continue;
          }
        }
        // $this->generateDeriverByEntityBundled($derivative, $entity_type_labels, $settings['target_type'], $bundle, $field_name, $entity_type_id_parent, $field_definition, $field_name_parent);
      }
    }
  }

  /**
   * Test if we can add the formatter field formatter.
   *
   * @return bool|null
   */
  private function isApplicableEntityReference($plugin_definition) {
    if (empty($plugin_definition['ui_patterns_field_formatter']) || empty($plugin_definition['ui_patterns_field_formatter']['field_definition'])) {
      $this->getLogger('ui_patterns_field_formatters')
        ->error('Missing ui_patterns_field_formatter data inside plugin definition %plugin_id', [
          '%plugin_id' => $this->getPluginId(),
        ]);
      return FALSE;
    }
    // Only available for slots.
    $source_name = 'props';
    if (!empty($plugin_definition['prop_types'][0]) && $plugin_definition['prop_types'][0] === 'slot') {
      return TRUE;
    }
    // $this->getPgetPropTypesDeriver($property_definition);
    if ($source_name === 'props') {
      return FALSE;
    }
    // We add field formatter only on the main property field.
    if (empty($plugin_definition['ui_patterns_field_formatter']['main_property'])) {
      return FALSE;
    }
    return TRUE;
  }

}
