<?php

namespace Drupal\Tests\ui_patterns_field_formatters\Kernel;

use Drupal\Component\Plugin\PluginBase;
use Drupal\Tests\node\Traits\NodeCreationTrait;
use Drupal\Tests\sdc\Traits\ComponentRendererTrait;
use Drupal\Tests\user\Traits\UserCreationTrait;

/**
 * @group ui_patterns_field_formatters
 * @SuppressWarnings(PHPMD.CamelCaseParameterName)
 * @SuppressWarnings(PHPMD.CamelCaseVariableName)
 * @SuppressWarnings(PHPMD.LongClassName)
 * @SuppressWarnings(PHPMD.LongVariable)
 */
class RenderTest extends UIPatternsFieldFormattersTestBase {

  use NodeCreationTrait {
    createNode as drupalCreateNode;
  }
  use UserCreationTrait {
    createUser as drupalCreateUser;
  }

  /**
   * @todo remove this after ui_patterns test.
   */
  protected $strictConfigSchema = FALSE;

  use ComponentRendererTrait;

  /**
   * Admin user.
   *
   * @var \Drupal\User\UserInterface
   */
  protected $adminUser;

  protected $source_id;

  /**
   * {@inheritdoc}
   */
  protected static $modules = [
    'datetime',
    'field',
    'filter',
    'node',
    'sdc',
    'system',
    'text',
    'ui_patterns',
    'ui_patterns_test',
    'ui_patterns_field_formatters',
    'user',
  ];

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();

    /* if (empty($field_definitions[$field_name])) {
    $this->getLogger('field')
    ->error('Field %field_name exists but is missing a corresponding field definition and may be misconfigured.', ['%field_name' => "$this->entityType.$bundle.$field_name"]);
    return FALSE;
    }*/
    // $this->drupalCreateUser();
    $this->installConfig('filter');
    $this->installSchema('system', ['sequences']);
    $this->installEntitySchema('date_format');
    $this->source_id = PluginBase::DERIVATIVE_SEPARATOR . $this->entityType->getEntityType()
      ->getBundleOf() . PluginBase::DERIVATIVE_SEPARATOR . $this->entityType->id() . PluginBase::DERIVATIVE_SEPARATOR . $this->field->getName() . PluginBase::DERIVATIVE_SEPARATOR . $this->property;
    // $this->setUpFieldDisplay();
  }

  /**
   * Set up the node with an field body.
   *
   * @return void
   *
   * @throws \Drupal\Core\Entity\EntityStorageException
   */
  private function setUpFieldDisplay(string $deriver_type, string $type, string $component_id, string $key_map, array $plugin, string $source_id) {
    /** @var \Drupal\Core\Entity\EntityDisplayRepositoryInterface $display_repository */
    $display_repository = $this->container->get('entity_display.repository');
    $settings = [
      'plugin' => $plugin,
      'source_id' => $source_id . $this->source_id,
    ];
    $ui_patterns = [
      'component_id' => $component_id,
      'variant_id' => 'primary',
      'slots' => [],
      'props' => [],
    ];
    if ($deriver_type === 'slots') {
      $settings['sources'] = [$settings];
    }
    $ui_patterns[$deriver_type][$key_map] = $settings;
    // Config field display.
    $display_repository->getViewDisplay($this->entityType->getEntityType()
      ->getBundleOf(), $this->entityType->id())
      ->setComponent('body', [
        'label' => 'hidden',
        'type' => $type,
        'region' => 'content',
        'settings' => [
          'ui_patterns' => $ui_patterns,
        ],
      ])
      ->save();
  }

  /**
   * @param string $deriver_type
   * @param string $component_id
   * @param string $key_map
   * @param array $plugin
   * @param string $source_id
   * @param string $expected_output
   *
   * @return void
   * @throws \Drupal\Core\Entity\EntityStorageException
   *
   * @dataProvider providerFieldMapping
   */
  public function testRender(string $deriver_type, string $type, string $component_id, string $key_map, array $plugin, string $source_id, string $expected_output) {
    // Create node page with body.
    // is_file('qsdsqd');.
    $this->setUpFieldDisplay($deriver_type, $type, $component_id, $key_map, $plugin, $source_id);
    $node = $this->drupalCreateNode();

    $node->set('body', ['this is first body', 'this is second body']);
    /** @var \Drupal\Core\Entity\EntityDisplayRepositoryInterface $display_repository */
    $display_repository = $this->container->get('entity_display.repository');
    $entity_display = $display_repository->getViewDisplay($this->entityType->getEntityType()
      ->getBundleOf(), $this->entityType->id());
    $entity_display->removeComponent('created');
    $entity_display->removeComponent('uid');
    $entity_display->removeComponent('links');
    $entity_display->removeComponent('title');
    $build = $entity_display->build($node);
    $renderer = $this->container->get('renderer');
    $renderer->renderRoot($build);
    // Echo $build['body']['#markup'];.
    $this->assertEquals($expected_output, $build['body']['#markup']);
  }

  /**
   * Callback data provider to test one slot, one prop.
   *
   * @return array[]
   */
  public function providerFieldMapping(): array {
    // @todo
    // avec field formatter.
    return [
      // Test 1.
      [
        // Slots without alter.
        'deriver_type' => 'slots',
        'type' => 'ui_patterns_each',
        'component_id' => 'ui_patterns_test:alert',
        'key_map' => 'heading',
        'plugin' => ['alter' => ['text' => '']],
        'source_id' => 'field_properties_source_slot',
        'expected' => <<<EXPECTED

            <div>  

<div data-component-id="ui_patterns_test:alert" class="alert-primary alert" role="alert">
    <h4 class="alert-heading">this is first body</h4>
    
  </div>
</div>
          <div>  

<div data-component-id="ui_patterns_test:alert" class="alert-primary alert" role="alert">
    <h4 class="alert-heading">this is second body</h4>
    
  </div>
</div>
      
EXPECTED,
      ],
      // Test 2.
      [
        // Slots without alter.
        'deriver_type' => 'slots',
        'type' => 'ui_patterns_all',
        'component_id' => 'ui_patterns_test:alert',
        'key_map' => 'heading',
        'plugin' => ['alter' => ['text' => '']],
        'source_id' => 'field_properties_source_slot',
        'expected' => <<<EXPECTED

            <div>  

<div data-component-id="ui_patterns_test:alert" class="alert-primary alert" role="alert">
    <h4 class="alert-heading">this is first bodythis is second body</h4>
    
  </div>
</div>
      
EXPECTED,
      ],
      // Test 3.
      [
        // Slots with alter.
        'deriver_type' => 'slots',
        'type' => 'ui_patterns_each',
        'component_id' => 'ui_patterns_test:alert',
        'key_map' => 'heading',
        'plugin' => [
          'alter' => [
            'alter_field' => '1',
            'text' => 'alter text {{heading}}',
          ],
        ],
        'source_id' => 'field_properties_source_slot',
        'expected' => <<<EXPECTED

            <div>  

<div data-component-id="ui_patterns_test:alert" class="alert-primary alert" role="alert">
    <h4 class="alert-heading">alter text this is first body</h4>
    
  </div>
</div>
          <div>  

<div data-component-id="ui_patterns_test:alert" class="alert-primary alert" role="alert">
    <h4 class="alert-heading">alter text this is second body</h4>
    
  </div>
</div>
      
EXPECTED,
      ],
      // Test 4.
      [
        'deriver_type' => 'props',
        'type' => 'ui_patterns_each',
        'component_id' => 'ui_patterns_test:simple_types_tests',
        'key_map' => 'string',
        'plugin' => ['alter' => ['text' => '']],
        'source_id' => 'field_properties_source_prop',
        'expected' => <<<EXPECTED

            <div><h3>string</h3>
this is first body
</div>
          <div><h3>string</h3>
this is second body
</div>
      
EXPECTED,
      ],
      // Test 5.
      [
        'deriver_type' => 'props',
        'type' => 'ui_patterns_all',
        'component_id' => 'ui_patterns_test:simple_types_tests',
        'key_map' => 'string',
        'plugin' => ['alter' => ['text' => '']],
        'source_id' => 'field_properties_source_prop',
        'expected' => <<<EXPECTED

            <div><h3>string</h3>
this is first body
</div>
      
EXPECTED,
      ],
      [
        'deriver_type' => 'props',
        'type' => 'ui_patterns_each',
        'component_id' => 'ui_patterns_test:simple_types_tests',
        'key_map' => 'string',
        'plugin' => [
          'alter' => [
            'alter_field' => '1',
            'text' => 'alter text {{string}}',
          ],
        ],
        'source_id' => 'field_properties_source_prop',
        'expected' => <<<EXPECTED

            <div><h3>string</h3>
alter text this is first body
</div>
          <div><h3>string</h3>
alter text this is second body
</div>
      
EXPECTED,
      ],
    ];
  }

}
