<?php

declare(strict_types=1);

namespace Drupal\ui_patterns_layouts\Plugin\Layout;

use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Layout\LayoutDefault;
use Drupal\Core\Plugin\PluginFormInterface;
use Drupal\ui_patterns\Form\ComponentFormBuilderTrait;

/**
 * Component Layout.
 *
 * @package Drupal\ui_patterns_layouts\Plugin\Layout
 */
class ComponentLayout extends LayoutDefault implements PluginFormInterface {

  use ComponentFormBuilderTrait;

  /**
   * Returns the region names.
   *
   * @return string[]
   *   The region names.
   */
  public function getRegionNames() {
    return $this->pluginDefinition->getRegionNames();
  }

  /**
   * Returns the region.
   *
   * @return array[]
   *   The regions.
   */
  public function getRegions() {
    return $this->pluginDefinition->getRegions();
  }

  /**
   * {@inheritdoc}
   */
  public function build(array $regions) {
    $build = $this->buildComponentRenderable($this->getPluginDefinition()->id());
    $build['#layout'] = $this;
    $regions = parent::build($regions);
    foreach ($this->getPluginDefinition()->getRegionNames() as $region_name) {
      if (array_key_exists($region_name, $regions)) {
        $build['#slots'][$region_name] = $regions[$region_name];
      }
    }
    return $build;
  }

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    return parent::defaultConfiguration() + $this->getComponentFormDefault();
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    $form = parent::buildConfigurationForm($form, $form_state);
    $component_id = $this->getPluginDefinition()->id();
    return $form + $this->buildComponentsForm($form_state, [], $component_id, FALSE, TRUE);
  }

  /**
   * {@inheritdoc}
   */
  public function validateConfigurationForm(array &$form, FormStateInterface $form_state) {
  }

  /**
   * {@inheritdoc}
   */
  public function submitConfigurationForm(array &$form, FormStateInterface $form_state):void {
    $this->submitComponentsForm($form_state);
    parent::submitConfigurationForm($form, $form_state);
  }

}
