<?php

declare(strict_types=1);

namespace Drupal\ui_patterns_legacy;

use Symfony\Component\Finder\Finder;
use Symfony\Component\Yaml\Yaml;

/**
 * Component writer.
 */
class ComponentWriter {

  /**
   * Write component definition to YML file.
   */
  public function writeDefinition(string $component_id, array $definition, string $extension_path): void {
    $path = realpath(".") . '/' . $extension_path . '/components/' . $component_id . '/';
    if (!is_dir($path)) {
      mkdir($path, 0775, TRUE);
    }
    $filename = $component_id . '.component.yml';
    $yaml = Yaml::dump($definition, 10, 2);
    file_put_contents($path . $filename, $yaml);
  }

  /**
   * Copy component assets to the new folder.
   */
  public function copyAssets(string $extension_path, array $legacy_definition): void {
    $component_id = $legacy_definition['id'];
    $source = realpath(".") . '/' . $legacy_definition['base path'];
    $target = realpath(".") . '/' . $extension_path . '/components/' . $component_id;
    $finder = new Finder();
    $finder->files()->notName('*.patterns.yml')->notName('*.pattern.yml')->notName('*.ui_patterns.yml')->notName('*.html.twig')->in($source);
    foreach ($finder as $file) {
      $path = str_replace($source, $target, $file->getPath());
      if (!is_dir($path)) {
        mkdir($path, 0775, TRUE);
      }
      $from = $file->getPathname();
      $to = str_replace($source, $target, $from);
      copy($from, $to);
    }
  }

  /**
   * Copy component templates to the new locations.
   */
  public function copyTemplates(string $extension_path, array $legacy_definition): void {
    $component_id = $legacy_definition['id'];
    // Copy component template to component/.
    $source = realpath(".") . '/' . $legacy_definition['base path'] . '/pattern-' . $component_id . '.html.twig';
    $target = realpath(".") . '/' . $extension_path . '/components/' . $component_id . '/' . $component_id . '.twig';
    if (file_exists($source)) {
      copy($source, $target);
    }
    else {
      $source = realpath(".") . '/' . $legacy_definition['base path'] . '/pattern-' . str_replace('_', '-', $component_id) . '.html.twig';
      if (file_exists($source)) {
        copy($source, $target);
      }
    }
    // @todo copy preview template somehwere else in templates/
    // @todo alert if variant templates are found
  }

}
