<?php

declare(strict_types=1);

namespace Drupal\ui_patterns_legacy\Element;

use Drupal\Core\Security\TrustedCallbackInterface;
use Drupal\ui_patterns_legacy\RenderableConverter;

/**
 * Renders a component story.
 */
class ComponentElementAlter implements TrustedCallbackInterface {

  /**
   * {@inheritdoc}
   */
  public static function trustedCallbacks() {
    return ['convert'];
  }

  /**
   * Convert legacy render element to SDC render element.
   */
  public function convert(array $element): array {
    $converter = new RenderableConverter();
    $theme = \Drupal::service('theme.manager')->getActiveTheme()->getName();
    $converter->setExtension($theme);
    $element = $converter->convertPattern($element, "#");
    return $this->addPreviewStory($element);
  }

  /**
   * Load preview.
   *
   * @param array $element
   *   Render array.
   *
   * @return array
   *   Render array.
   */
  protected function addPreviewStory(array $element): array {
    $manager = \Drupal::service('plugin.manager.sdc');
    $component = $manager->getDefinition($element["#component"]);
    if (!isset($component["stories"])) {
      return $element;
    }
    if (empty($component["stories"])) {
      return $element;
    }
    $element["#story"] = self::getStoryId($component["stories"]);
    $element_alterer = \Drupal::service('ui_patterns.component_element_alter');
    $element["#slots"] = $element_alterer->processStoriesSlots($element["#slots"] ?? []);
    return $element;
  }

  /**
   * Get story ID.
   */
  private function getStoryId(array $stories): string {
    // In UI Patterns 1.x, there was only one story by component, called
    // "preview".
    if (array_key_exists("preview", $stories)) {
      return "preview";
    }
    return array_key_first($stories);
  }

}
