<?php

declare(strict_types=1);

namespace Drupal\ui_patterns_library\Element;

use Drupal\Core\Security\TrustedCallbackInterface;

/**
 * Renders a component story.
 */
class ComponentElementAlter implements TrustedCallbackInterface {

  /**
   * {@inheritdoc}
   */
  public static function trustedCallbacks() {
    return ['alter'];
  }

  /**
   * Alter SDC component element.
   */
  public function alter(array $element) {
    $element = $this->loadStory($element);
    return $element;
  }

  /**
   * Load story from component definition.
   */
  protected function loadStory(array $element): array {
    if (!isset($element["#story"])) {
      return $element;
    }
    $story_id = $element["#story"];
    $manager = \Drupal::service('plugin.manager.sdc');
    $component = $manager->getDefinition($element["#component"]);
    if (!isset($component["stories"])) {
      return $element;
    }
    if (!isset($component["stories"][$story_id])) {
      return $element;
    }
    $story = $component["stories"][$story_id];
    $slots = array_merge($element["#slots"] ?? [], $story["slots"] ?? []);
    $element_alterer = \Drupal::service('ui_patterns.component_element_alter');
    $element["#slots"] = $element_alterer->processStoriesSlots($slots);
    $element["#props"] = array_merge($element["#props"] ?? [], $story["props"] ?? []);
    return $element;
  }

}
