<?php

declare(strict_types=1);

namespace Drupal\ui_patterns_views\Plugin\views\style;

use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Url;
use Drupal\ui_patterns\Form\ComponentSettingsFormBuilderTrait;
use Drupal\views\Plugin\views\style\StylePluginBase;

/**
 * Style plugin to render items in a pattern field.
 *
 * @ingroup views_style_plugins
 *
 * @ViewsStyle(
 *     id = "ui_patterns",
 *     title = @Translation("Component (UI Patterns)"),
 *     help = @Translation("Displays views with UI components."),
 *     theme = "view--pattern",
 *     display_types = {"normal"}
 * )
 */
class ComponentStyle extends StylePluginBase {

  use ComponentSettingsFormBuilderTrait;

  /**
   * {@inheritdoc}
   */
  protected $usesRowPlugin = TRUE;

  /**
   * {@inheritdoc}
   */
  protected $usesOptions = TRUE;

  /**
   * {@inheritdoc}
   */
  protected function defineOptions(): array {
    return parent::defineOptions() + ['ui_patterns' => ['default' => self::getComponentFormDefault()['ui_patterns']]];
  }

  /**
   * {@inheritdoc}
   */
  public function buildOptionsForm(&$form, FormStateInterface $form_state) {
    $ui_patterns_form = $this->componentSettingsForm($form, $form_state);
    $form['ui_patterns'] = $ui_patterns_form['ui_patterns'];
    parent::buildOptionsForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function getComponentSettings(): array {
    return $this->options;
  }

  /**
   * {@inheritdoc}
   */
  protected function getAjaxUrl(FormStateInterface $form_state): ?Url {
    return views_ui_build_form_url($form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function render(): array {
    return [];
  }

}
