<?php

namespace Drupal\ui_patterns\Element;

use Drupal\Core\Plugin\Component;
use Drupal\Core\Render\Element\FormElementBase;

/**
 * Base class for components forms.
 */
abstract class ComponentFormBase extends FormElementBase {

  /**
   * Expand each ajax element with ajax urls.
   *
   * @param array $element
   *   The ajax element.
   *
   * @return array
   *   The extended ajax form.
   */
  protected static function expandAjax(array $element): array {
    $url = $element['#ajax_url'] ?? NULL;
    if (isset($element['#ajax']) && $url) {
      $element['#ajax']['url'] = $url;
    }
    return $element;
  }

  /**
   * Helper function to return the component.
   */
  protected static function getComponent(array $element): Component | NULL {
    $component_id = $element['#default_value']['component_id'] ?? $element['#component_id'] ?? NULL;
    /** @var \Drupal\Core\Theme\ComponentPluginManager $component_plugin_manager */
    $component_plugin_manager = \Drupal::service("plugin.manager.sdc");
    return $component_id ? $component_plugin_manager->find($component_id) : NULL;
  }

  /**
   * Returns the parents belonging to the form builder.
   *
   * @param array $parents
   *   The form_state '#parents' value.
   *
   * @return array
   *   The parents to locate the form builder.
   */
  protected static function getComponentFormStateParents(array $parents): array {
    $needle = array_search('component_form', $parents);
    return array_slice($parents, 0, $needle);
  }

}
