<?php

declare(strict_types=1);

namespace Drupal\ui_patterns\Plugin\UiPatterns\PropType;

use Drupal\ui_patterns\PropTypePluginBase;

/**
 * Provides a 'number' PropType.
 *
 * @PropType(
 *   id = "number",
 *   label = @Translation("Number"),
 *   description = @Translation("Either integers or floating point numbers."),
 *   priority = 1,
 *   default_source = "number",
 *   schema = {
 *     "type": {"number", "integer"}
 *   }
 * )
 */
class NumberPropType extends PropTypePluginBase {

  /**
   * {@inheritdoc}
   */
  public function getSummary(array $definition): array {
    $summary = parent::getSummary($definition);
    if (isset($definition['type']) && $definition['type'] === "integer") {
      $summary[] = $this->t("Integers only");
    }
    if (isset($definition['minimum'])) {
      $summary[] = $this->t("Minimum: @length", ["@length" => $definition['minimum']]);
    }
    if (isset($definition['maximum'])) {
      $summary[] = $this->t("Maximum: @length", ["@length" => $definition['minimum']]);
    }
    return $summary;
  }

}
