<?php

declare(strict_types=1);

namespace Drupal\ui_patterns\Plugin\UiPatterns\PropType;

use Drupal\Component\Render\MarkupInterface;
use Drupal\Core\Render\RenderableInterface;
use Drupal\ui_patterns\PropTypePluginBase;

/**
 * Provides a 'Slot' PropType.
 *
 * @PropType(
 *   id = "slot",
 *   label = @Translation("Slot"),
 *   description = @Translation("A placeholder inside a component that can be filled with renderables."),
 *   priority = 10,
 *   default_source = "wysiwyg",
 *   schema = {
 *   }
 * )
 */
class SlotPropType extends PropTypePluginBase {

  /**
   * {@inheritdoc}
   */
  public static function normalize(mixed $value): mixed {
    if (is_object($value)) {
      return self::convertObject($value);
    }
    if (is_string($value)) {
      return ['#markup' => $value];
    }
    return $value;
  }

  /**
   * Convert PHP objects to render array.
   */
  protected static function convertObject(object $value): array {
    if ($value instanceof RenderableInterface) {
      $value = $value->toRenderable();
    }
    if ($value instanceof MarkupInterface) {
      return [
        '#markup' => (string) $value,
      ];
    }
    elseif ($value instanceof \Stringable) {
      return [
        '#plain_text' => (string) $value,
      ];
    }
    return [];
  }

}
