<?php

declare(strict_types=1);

namespace Drupal\ui_patterns\Plugin\UiPatterns\PropType;

use Drupal\ui_patterns\PropTypePluginBase;

/**
 * Provides a 'string' PropType.
 *
 * @PropType(
 *   id = "string",
 *   label = @Translation("String"),
 *   description = @Translation("Strings of text. May contain Unicode characters."),
 *   priority = 1,
 *   default_source = "textfield",
 *   schema = {
 *     "type": "string"
 *   }
 * )
 */
class StringPropType extends PropTypePluginBase {

  /**
   * {@inheritdoc}
   */
  public function getSummary(array $definition): array {
    $summary = parent::getSummary($definition);
    if (isset($definition['maxLength'])) {
      $summary[] = $this->t("Max length: @length", ["@length" => $definition['maxLength']]);
    }
    if (isset($definition['minLength'])) {
      $summary[] = $this->t("Min length: @length", ["@length" => $definition['minLength']]);
    }
    return $summary;
  }

}
