<?php

declare(strict_types=1);

namespace Drupal\ui_patterns\Plugin\UiPatterns\Source;

use Drupal\Core\Form\FormStateInterface;
use Drupal\ui_patterns\SourcePluginBase;

/**
 * Plugin implementation of the source.
 *
 * @Source(
 *   id = "checkboxes",
 *   label = @Translation("Checkboxes"),
 *   description = @Translation("A set of checkboxes."),
 *   prop_types = {
 *     "enum_list"
 *   }
 * )
 */
class CheckboxesWidget extends SourcePluginBase {

  /**
   * {@inheritdoc}
   */
  public function getPropValue(): mixed {
    $value = $this->getSetting('value') ?? [];
    $value = is_scalar($value) ? [$value] : $value;
    return array_filter($value);
  }

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state): array {
    $options = [];
    foreach ($this->propDefinition['items']['enum'] as $item) {
      $options[$item] = $item;
    }
    return [
      'value' => [
        '#type' => 'checkboxes',
        '#default_value' => $this->getSetting('value') ?? [],
        "#options" => $options,
      ],
    ];
  }

}
