<?php

declare(strict_types=1);

namespace Drupal\ui_patterns\Plugin\UiPatterns\Source;

use Drupal\Core\Form\FormStateInterface;
use Drupal\ui_patterns\SourcePluginBase;

/**
 * Plugin implementation of the source.
 *
 * @Source(
 *   id = "list_textarea",
 *   label = @Translation("Textarea for list"),
 *   description = @Translation("One item by line."),
 *   prop_types = {
 *     "list"
 *   }
 * )
 */
class ListTextareaWidget extends SourcePluginBase {

  /**
   * {@inheritdoc}
   */
  public function getPropValue(): mixed {
    $long_text = $this->getSetting('value') ?? '';
    return preg_split("/\r\n|\n|\r/", $long_text);
  }

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state): array {
    $items = $this->getSetting('value');
    if (is_array($items)) {
      $items = implode("\r", $items);
    }
    return [
      'value' => [
        '#type' => 'textarea',
        '#default_value' => $items,
        "#description" => $this->t("One item by line"),
      ],
    ];
  }

}
