<?php

declare(strict_types=1);

namespace Drupal\ui_patterns\Plugin\UiPatterns\Source;

use Drupal\Core\Form\FormStateInterface;
use Drupal\ui_patterns\SourcePluginBase;

/**
 * Plugin implementation of the source.
 *
 * @Source(
 *   id = "number",
 *   label = @Translation("Number"),
 *   description = @Translation("Numeric input, with special numeric validation."),
 *   prop_types = {
 *     "number"
 *   }
 * )
 */
class NumberWidget extends SourcePluginBase {

  /**
   * {@inheritdoc}
   */
  public function getPropValue(): mixed {
    $value = $this->getSetting('value');
    // Add 0 to automatically cast to a float OR an integer.
    if (empty($value)) {
      return $value;
    }
    return $value + 0;
  }

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state): array {
    $build = [];
    $build['value'] = [
      '#type' => 'number',
      '#default_value' => $this->getSetting('value'),
      '#step' => 0.01,
    ];
    if ($this->propDefinition["type"] === "integer") {
      $build['value']['#step'] = 1;
    }
    // Because of SDC's ComponentMetadata::parseSchemaInfo() which is adding
    // "object" type to all props to "allows deferring rendering in Twig to the
    // render pipeline". Remove it as soon as this weird mechanism is removed
    // from SDC.
    if (in_array("integer", $this->propDefinition["type"])) {
      $build['value']['#step'] = 1;
    }
    if (isset($this->propDefinition["minimum"])) {
      $build['value']['#min'] = $this->propDefinition["minimum"];
    }
    if (isset($this->propDefinition["maximum"])) {
      $build['value']['#max'] = $this->propDefinition["maximum"];
    }
    return $build;
  }

}
