<?php

declare(strict_types=1);

namespace Drupal\ui_patterns\Plugin\UiPatterns\Source;

use Drupal\Core\Form\FormStateInterface;
use Drupal\ui_patterns\SourcePluginBase;

/**
 * Plugin implementation of the source.
 *
 * @Source(
 *   id = "select",
 *   label = @Translation("Select"),
 *   description = @Translation("A drop-down menu or scrolling selection box."),
 *   prop_types = {
 *     "enum"
 *   }
 * )
 */
class SelectWidget extends SourcePluginBase {

  /**
   * {@inheritdoc}
   */
  public function getPropValue(): mixed {
    return $this->getSetting('value');
  }

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state): array {
    $build = [];
    $options = [];
    foreach ($this->propDefinition['enum'] as $item) {
      $options[$item] = ucwords($item);
    }
    $build['value'] = [
      '#type' => 'select',
      '#default_value' => $this->getSetting('value'),
      "#options" => $options,
    ];
    // With Firefox, autocomplete may override #default_value.
    // https://drupal.stackexchange.com/questions/257732/default-value-not-working-in-select-field
    $build['value']['#attributes']['autocomplete'] = 'off';
    return $build;
  }

}
