<?php

declare(strict_types=1);

namespace Drupal\ui_patterns\Plugin\UiPatterns\Source;

use Drupal\Core\Form\FormStateInterface;
use Drupal\ui_patterns\SourcePluginBase;

/**
 * Plugin implementation of the source.
 *
 * @Source(
 *   id = "textfield",
 *   label = @Translation("Textfield"),
 *   description = @Translation("One-line text field."),
 *   prop_types = {
 *     "string",
 *     "machine_name"
 *   }
 * )
 */
class TextfieldWidget extends SourcePluginBase {

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state): array {
    $build = [];
    $build['value'] = [
      '#type' => 'textfield',
      '#default_value' => $this->getSetting('value'),
    ];
    $description = [];
    if (isset($this->propDefinition["pattern"])) {
      $build['value']['#pattern'] = $this->propDefinition["pattern"];
      $description[] = $this->t("Constraint: @pattern", ["@pattern" => $this->propDefinition["pattern"]]);
    }
    if (isset($this->propDefinition["maxLength"])) {
      $build['value']['#maxlength'] = $this->propDefinition["maxLength"];
      $build['value']['#size'] = $this->propDefinition["maxLength"];
      $description[] = $this->t("Max length: @length", ["@length" => $this->propDefinition["maxLength"]]);
    }
    if (!isset($this->propDefinition["pattern"]) && isset($this->propDefinition["minLength"])) {
      // @todo Cover also the use case pattern + minLength.
      $build['value']['#pattern'] = "^.{" . $this->propDefinition["minLength"] . ",}$";
      $description[] = $this->t("Min length: @length", ["@length" => $this->propDefinition["minLength"]]);
    }
    $build['value']["#description"] = implode("; ", $description);
    return $build;
  }

}
