<?php

declare(strict_types=1);

namespace Drupal\ui_patterns\Plugin\UiPatterns\Source;

use Drupal\Core\Form\FormStateInterface;
use Drupal\ui_patterns\SourcePluginBase;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Plugin implementation of the source.
 *
 * @Source(
 *   id = "token",
 *   label = @Translation("Token"),
 *   description = @Translation("Placeholder variables, replaced before display."),
 *   prop_types = {
 *     "string"
 *   },
 *   context_definitions = {
 *     "ui_patterns:entity" = @ContextDefinition("entity",
 *       label = @Translation("Entity"),
 *       required = FALSE
 *     )
 *   }
 * )
 */
class TokenSource extends SourcePluginBase {

  /**
   * The token manager.
   *
   * @var \Drupal\Core\Utility\Token
   */
  protected $tokenManager;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    $plugin = parent::create(
      $container,
      $configuration,
      $plugin_id,
      $plugin_definition
    );
    $plugin->tokenManager = $container->get('token');
    return $plugin;
  }

  /**
   * {@inheritdoc}
   */
  public function getPropValue(): mixed {
    $value = $this->getSetting('value');
    if (!is_scalar($value)) {
      return "";
    }
    $value = (string) $value;
    $entity = $this->getContextValue('ui_patterns:entity');
    $token_data = [];
    if ($entity !== NULL) {
      $token_data[$entity->getEntityTypeId()] = $entity;
    }
    return $this->tokenManager->replacePlain($value, $token_data, ['clear' => TRUE]);
  }

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state): array {
    return [
      'value' => [
        '#type' => 'textfield',
        '#default_value' => $this->getSetting('value'),
        // Tokens always start with a [ and end with a ].
        '#pattern' => '^\[.+\]$',
      ],
      'context_mapping' => $this->addContextAssignmentElement($this, $this->gatheredContexts),
    ];
  }

}
