<?php

declare(strict_types=1);

namespace Drupal\ui_patterns;

use Drupal\Core\Cache\CacheBackendInterface;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\Plugin\DefaultPluginManager;
use Drupal\ui_patterns\Annotation\PropTypeAdapter;

/**
 * PropTypeAdapter plugin manager.
 */
final class PropTypeAdapterPluginManager extends DefaultPluginManager implements SchemaGuesserInterface {

  /**
   * Constructs the object.
   */
  public function __construct(\Traversable $namespaces, CacheBackendInterface $cache_backend, ModuleHandlerInterface $module_handler) {
    parent::__construct('Plugin/UiPatterns/PropTypeAdapter', $namespaces, $module_handler, PropTypeAdapterInterface::class, PropTypeAdapter::class);
    $this->alterInfo('prop_type_adapter_info');
    $this->setCacheBackend($cache_backend, 'prop_type_adapter_plugins');
  }

  /**
   * {@inheritdoc}
   */
  public function guessFromSchema(array $prop_schema): ?PropTypeAdapterInterface {
    $definitions = $this->getDefinitions();
    $compatibilityChecker = \Drupal::service('ui_patterns.schema_compatibility_checker');
    foreach ($definitions as $definition) {
      if ($compatibilityChecker->isCompatible($prop_schema, $definition['schema'])) {
        return $this->createInstance($definition['id'], []);
      }
    }
    return NULL;
  }

}
