<?php

namespace Drupal\ui_patterns;

use Drupal\Component\Plugin\PluginBase;
use Drupal\Core\StringTranslation\StringTranslationTrait;

/**
 * Base class for prop_type plugins.
 */
abstract class PropTypePluginBase extends PluginBase implements PropTypeInterface {

  use StringTranslationTrait;

  /**
   * {@inheritdoc}
   */
  public function label(): string {
    // Cast the label to a string since it is a TranslatableMarkup object.
    return (string) $this->pluginDefinition['label'];
  }

  /**
   * {@inheritdoc}
   */
  public function convertFrom(string $prop_type, mixed $value): mixed {
    // @todo implement
    return NULL;
  }

  /**
   * Get default source ID.
   */
  public function getDefaultSourceId(): string {
    return $this->pluginDefinition['default_source'] ?? '';
  }

  /**
   * {@inheritdoc}
   */
  public function getSchema(): array {
    $schema = $this->pluginDefinition['schema'];
    return (array) $schema;
  }

  /**
   * {@inheritdoc}
   */
  public function getSummary(array $definition): array {
    $summary = [];
    if (isset($definition['description'])) {
      $summary[] = $definition['description'];
    }
    if (isset($definition['enum'])) {
      $summary[] = $this->t("Values: @values", ["@values" => implode(", ", $definition['enum'])]);
    }
    if (isset($definition['default']) && is_scalar($definition['default'])) {
      $summary[] = $this->t("Default: @default", ["@default" => $definition['default']]);
    }
    if (isset($definition['default']) && is_array($definition['default'])) {
      $summary[] = $this->t("Default: @default", ["@default" => implode(", ", $definition['default'])]);
    }
    return $summary;
  }

  /**
   * {@inheritdoc}
   */
  public static function normalize(mixed $value): mixed {
    return $value;
  }

}
