<?php

declare(strict_types=1);

namespace Drupal\Tests\ui_patterns\Kernel;

use Drupal\KernelTests\KernelTestBase;
use Drupal\ui_patterns\SourcePluginBase;

/**
 * Test SourcePluginManager.
 *
 * @group ui_patterns
 */
final class SourcePluginManagerTest extends KernelTestBase {

  /**
   * {@inheritdoc}
   */
  protected static $modules = ['ui_patterns', 'ui_patterns_test'];

  /**
   * Validate source.
   */
  private function validateSource(SourcePluginBase $source) {
    self::assertNotNull($source);
    self::assertInstanceOf(SourcePluginBase::class, $source);
    self::assertNotNull($source->getPropId());
    self::assertNotNull($source->getPropDefinition());
  }

  /**
   * Test callback.
   */
  public function testGetSourcePlugins(): void {
    $source_plugin_manager = \Drupal::service('plugin.manager.ui_patterns_source');
    $source_ids = array_keys($source_plugin_manager->getDefinitionsForPropType('string'));
    $configuration = [
      'prop_id' => 'test',
      'prop_definition' => ['title' => 'test title'],
      'form_value' => [],
    ];
    $source_ids = array_combine($source_ids, $source_ids);
    $sources = $source_plugin_manager->createInstances($source_ids, $configuration);
    $plugin_ids = [];
    /** @var \Drupal\ui_patterns\SourcePluginBase $source */
    foreach ($sources as $source) {
      $plugin_ids[$source->getPluginId()] = $source;
      $this->validateSource($source);
    }
    self::assertFalse(isset($plugin_ids['context_foo']));
    self::assertTrue(isset($plugin_ids['foo']));
    self::assertGreaterThan(1, $sources);
  }

}
