<?php

declare(strict_types=1);

namespace Drupal\Tests\ui_patterns\Unit;

use Drupal\Component\Serialization\Yaml;
use Drupal\Tests\UnitTestCase;
use Drupal\ui_patterns\SchemaManager\Canonicalizer;
use Drupal\ui_patterns\SchemaManager\CompatibilityChecker;

/**
 * Test description.
 *
 * @group ui_patterns
 */
final class SchemaCompatibilityCheckerTest extends UnitTestCase {

  /**
   * Tests something.
   *
   * @dataProvider provideIsCompatibleData
   */
  public function testIsCompatible(array $checked_schema, array $reference_schema, bool $expected_result): void {
    $validator = new CompatibilityChecker(new Canonicalizer());
    self::assertEquals($expected_result, $validator->isCompatible($checked_schema, $reference_schema));
  }

  /**
   * Provide data for SchemaCompatibilityChecker::isComaptible() method.
   */
  public function provideIsCompatibleData() {
    $data = [];
    $sources = Yaml::decode(file_get_contents(__DIR__ . "/schema_compatibility_checker_data.yml"));
    foreach ($sources as $source) {
      foreach ($source["tests"] as $test) {
        $data[] = [
          $test["schema"],
          $source["schema"],
          (bool) $test["result"],
        ];
      }
    };
    return $data;
  }

}
